/*
  * JBoss, Home of Professional Open Source
  * Copyright 2005, JBoss Inc., and individual contributors as indicated
  * by the @authors tag. See the copyright.txt in the distribution for a
  * full listing of individual contributors.
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This software is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this software; if not, write to the Free
  * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
  */
package org.jboss.test.messaging.jms.message;


import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.TextMessage;

import org.jboss.jms.message.MessageProxy;

/**
 * A test that sends/receives text messages to the JMS provider and verifies their integrity.
 *
 * @author <a href="mailto:ovidiu@feodorov.com">Ovidiu Feodorov</a>
 * @version <tt>$Revision: 8385 $</tt>
 *
 * $Id: TextMessageTest.java 8385 2011-07-15 10:13:23Z gaohoward $
 */
public class TextMessageTest extends MessageTestBase
{
   // Constants -----------------------------------------------------

   // Static --------------------------------------------------------
   
   // Attributes ----------------------------------------------------

   // Constructors --------------------------------------------------

   public TextMessageTest(String name)
   {
      super(name);
   }

   // Public --------------------------------------------------------

   public void setUp() throws Exception
   {
      super.setUp();
      message = session.createTextMessage();
   }

   public void tearDown() throws Exception
   {
      message = null;
      super.tearDown();
   }

   public void testClearProperties() throws Exception
   {
      ((TextMessage)message).setText("something");
      queueProd.send(message);

      TextMessage rm = (TextMessage)queueCons.receive();

      rm.clearProperties();

      assertEquals("something", rm.getText());
   }
   
   //JBMESSAGING-1888
   public void testMessageSerializable() throws Exception
   {
      queueProd.send(message);
      
      log.debug("Message sent");

      Message r = queueCons.receive();
      
      log.debug("Message received");
      
      ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
      ObjectOutputStream cloneOut = new ObjectOutputStream(byteOut);
      cloneOut.writeObject(r);
      cloneOut.close();
      ObjectInputStream inputArray = new ObjectInputStream (new ByteArrayInputStream(byteOut.toByteArray()));
      Message clone = (Message)inputArray.readObject();      
      log.info("cloned message: " + clone);
   }
   
   // Protected -----------------------------------------------------

   protected void prepareMessage(Message m) throws JMSException
   {
      super.prepareMessage(m);

      TextMessage tm = (TextMessage)m;
      tm.setText("this is the payload");
   }

   protected void assertEquivalent(Message m, int mode, boolean redelivery) throws JMSException
   {
      super.assertEquivalent(m, mode, redelivery);

      TextMessage tm = (TextMessage)m;
      assertEquals("this is the payload", tm.getText());
   }
}
