/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.test.messaging.util;

import org.jboss.messaging.util.XMLUtil;
import org.jboss.test.messaging.MessagingTestCase;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;



/**
 * @author <a href="mailto:ovidiu@feodorov.com">Ovidiu Feodorov</a>
 * @version <tt>$Revision: 7921 $</tt>
 */
public class XMLUtilTest extends MessagingTestCase
{
   // Constructors --------------------------------------------------

   public XMLUtilTest(String name)
   {
      super(name);
   }

   // Public --------------------------------------------------------

   public void setUp() throws Exception
   {
      super.setUp();
   }

   public void tearDown() throws Exception
   {
      super.tearDown();
   }

   public void testGetTextContext_1() throws Exception
   {
      String document = "<blah>foo</blah>";

      Element e = XMLUtil.stringToElement(document);

      assertEquals("foo", XMLUtil.getTextContent(e));
   }

   public void testGetTextContext_2() throws Exception
   {
      String document = "<blah someattribute=\"somevalue\">foo</blah>";

      Element e = XMLUtil.stringToElement(document);

      assertEquals("foo", XMLUtil.getTextContent(e));
   }

   public void testGetTextContext_3() throws Exception
   {
      String document = "<blah someattribute=\"somevalue\"><a/></blah>";

      Element e = XMLUtil.stringToElement(document);

      String s = XMLUtil.getTextContent(e);

      Element subelement = XMLUtil.stringToElement(s);

      assertEquals("a", subelement.getNodeName());
   }

   public void testGetTextContext_4() throws Exception
   {
      String document = "<blah someattribute=\"somevalue\"><a></a></blah>";

      Element e = XMLUtil.stringToElement(document);

      String s = XMLUtil.getTextContent(e);

      Element subelement = XMLUtil.stringToElement(s);

      assertEquals("a", subelement.getNodeName());
   }

   public void testGetTextContext_5() throws Exception
   {
      String document = "<blah someattribute=\"somevalue\"><a><b/></a></blah>";

      Element e = XMLUtil.stringToElement(document);

      String s = XMLUtil.getTextContent(e);

      Element subelement = XMLUtil.stringToElement(s);

      assertEquals("a", subelement.getNodeName());
      NodeList nl = subelement.getChildNodes();

      // try to find <b>
      boolean found = false;
      for(int i = 0; i < nl.getLength(); i++)
      {
         Node n = nl.item(i);
         if ("b".equals(n.getNodeName()))
         {
            found = true;
         }
      }
      assertTrue(found);
   }


   public void testEquivalent_1() throws Exception
   {
      String s = "<a/>";
      String s2 = "<a/>";

      XMLUtil.assertEquivalent(XMLUtil.stringToElement(s), XMLUtil.stringToElement(s2));
   }

   public void testEquivalent_2() throws Exception
   {
      String s = "<a></a>";
      String s2 = "<a/>";

      XMLUtil.assertEquivalent(XMLUtil.stringToElement(s), XMLUtil.stringToElement(s2));
   }

   public void testEquivalent_3() throws Exception
   {
      String s = "<a attr1=\"val1\" attr2=\"val2\"/>";
      String s2 = "<a attr2=\"val2\"/>";

      try
      {
         XMLUtil.assertEquivalent(XMLUtil.stringToElement(s), XMLUtil.stringToElement(s2));
         fail("this should throw exception");
      }
      catch(IllegalArgumentException e)
      {
         // OK
         log.debug("STACK TRACE", e);
      }
   }

   public void testEquivalent_4() throws Exception
   {
      String s = "<a attr1=\"val1\" attr2=\"val2\"/>";
      String s2 = "<a attr2=\"val2\" attr1=\"val1\"/>";

      XMLUtil.assertEquivalent(XMLUtil.stringToElement(s), XMLUtil.stringToElement(s2));
   }

   public void testEquivalent_5() throws Exception
   {
      String s = "<a><b/></a>";
      String s2 = "<a><b/></a>";

      XMLUtil.assertEquivalent(XMLUtil.stringToElement(s), XMLUtil.stringToElement(s2));
   }

   public void testEquivalent_6() throws Exception
   {
      String s = "<enclosing><a attr1=\"val1\" attr2=\"val2\"/></enclosing>";
      String s2 = "<enclosing><a attr2=\"val2\" attr1=\"val1\"/></enclosing>";

      XMLUtil.assertEquivalent(XMLUtil.stringToElement(s), XMLUtil.stringToElement(s2));
   }

   public void testEquivalent_7() throws Exception
   {
      String s = "<a><b/><c/></a>";
      String s2 = "<a><c/><b/></a>";

      try
      {
         XMLUtil.assertEquivalent(XMLUtil.stringToElement(s), XMLUtil.stringToElement(s2));
         fail("this should throw exception");
      }
      catch(IllegalArgumentException e)
      {
         // OK
         log.debug("STACK TRACE", e);
      }
   }

   public void testEquivalent_8() throws Exception
   {
      String s = "<a><!-- some comment --><b/><!--some other comment --><c/><!-- blah --></a>";
      String s2 = "<a><b/><!--blah blah--><c/></a>";

      XMLUtil.assertEquivalent(XMLUtil.stringToElement(s), XMLUtil.stringToElement(s2));
   }

   public void testElementToString_1() throws Exception
   {
      String s = "<a b=\"something\">somethingelse</a>";
      Element e = XMLUtil.stringToElement(s);
      String tostring = XMLUtil.elementToString(e);
      Element convertedAgain = XMLUtil.stringToElement(tostring);
      XMLUtil.assertEquivalent(e, convertedAgain);
   }

   public void testElementToString_2() throws Exception
   {
      String s = "<a b=\"something\"></a>";
      Element e = XMLUtil.stringToElement(s);
      String tostring = XMLUtil.elementToString(e);
      Element convertedAgain = XMLUtil.stringToElement(tostring);
      XMLUtil.assertEquivalent(e, convertedAgain);
   }

   public void testElementToString_3() throws Exception
   {
      String s = "<a b=\"something\"/>";
      Element e = XMLUtil.stringToElement(s);
      String tostring = XMLUtil.elementToString(e);
      Element convertedAgain = XMLUtil.stringToElement(tostring);
      XMLUtil.assertEquivalent(e, convertedAgain);
   }

   public void testElementToString_4() throws Exception
   {
      String s = "<a><![CDATA[somedata]]></a>";
      Element e = XMLUtil.stringToElement(s);
      String tostring = XMLUtil.elementToString(e);
      Element convertedAgain = XMLUtil.stringToElement(tostring);
      XMLUtil.assertEquivalent(e, convertedAgain);
   }

   //https://jira.jboss.org/jira/browse/JBMESSAGING-1632
   public void testDoReplace() throws Exception
   {
      String existingProp = System.getProperty("jboss.messaging.datachanneludpaddress");
      System.clearProperty("jboss.messaging.datachanneludpaddress");
      
      String s1 = "${jboss.messaging.datachanneludpaddress:228.6.6.6}";
      String s2 = "${jboss.messaging.datachanneludpaddress,jboss.partition.udpGroup:228.6.6.6}";
      String s3 = "${jboss.partition.udpGroup,jboss.messaging.datachanneludpaddress:228.6.6.6}";
      String s4 = "${jboss.messaging.datachanneludpaddress,jboss.partition.udpGroup,jboss.partition.udpGroup1:228.6.6.6}";
      String s5 = "${jboss.partition.udpGroup1,jboss.partition.udpGroup,jboss.messaging.datachanneludpaddress:228.6.6.6}";
      String s6 = "${jboss.partition.udpGroup1,jboss.partition.udpGroup,jboss.messaging.datachanneludpaddress,jboss.partition.udpGroup2:228.6.6.6}";
      String s7 = "absolutely nothing";
      String s8 = "${jboss.partition.udpGroup1,jboss.partition.udpGroup,jboss.partition.udpGroup2:228.6.6.6}";
      
      String s1_val0 = XMLUtil.doReplace(s1, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s2_val0 = XMLUtil.doReplace(s2, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s3_val0 = XMLUtil.doReplace(s3, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s4_val0 = XMLUtil.doReplace(s4, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s5_val0 = XMLUtil.doReplace(s5, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s6_val0 = XMLUtil.doReplace(s6, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s7_val0 = XMLUtil.doReplace(s7, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s8_val0 = XMLUtil.doReplace(s8, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      
      log.info("s1_val0: " + s1_val0);
      log.info("s2_val0: " + s2_val0);
      log.info("s3_val0: " + s3_val0);
      log.info("s4_val0: " + s4_val0);
      log.info("s5_val0: " + s5_val0);
      log.info("s6_val0: " + s6_val0);
      log.info("s7_val0: " + s7_val0);
      log.info("s8_val0: " + s8_val0);

      assertEquals("228.6.6.6", s1_val0);
      assertEquals("228.6.6.6", s2_val0);
      assertEquals("228.6.6.6", s3_val0);
      assertEquals("228.6.6.6", s4_val0);
      assertEquals("228.6.6.6", s5_val0);
      assertEquals("228.6.6.6", s6_val0);
      assertEquals(s7, s7_val0);
      assertEquals(s8, s8_val0);

      System.setProperty("jboss.messaging.datachanneludpaddress", "222222");
      
      String s1_val = XMLUtil.doReplace(s1, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s2_val = XMLUtil.doReplace(s2, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s3_val = XMLUtil.doReplace(s3, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s4_val = XMLUtil.doReplace(s4, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s5_val = XMLUtil.doReplace(s5, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s6_val = XMLUtil.doReplace(s6, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s7_val = XMLUtil.doReplace(s7, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s8_val = XMLUtil.doReplace(s8, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      
      log.info("s1_val: " + s1_val);
      log.info("s2_val: " + s2_val);
      log.info("s3_val: " + s3_val);
      log.info("s4_val: " + s4_val);
      log.info("s5_val: " + s5_val);
      log.info("s6_val: " + s6_val);
      log.info("s7_val: " + s7_val);
      log.info("s8_val: " + s8_val);

      assertEquals("222222", s1_val);
      assertEquals("222222", s2_val);
      assertEquals("222222", s3_val);
      assertEquals("222222", s4_val);
      assertEquals("222222", s5_val);
      assertEquals("222222", s6_val);
      assertEquals(s7, s7_val);
      assertEquals(s8, s8_val);

      System.setProperty("jboss.messaging.datachanneludpaddress", "ff0e::1:2:3");
      
      String s1_val1 = XMLUtil.doReplace(s1, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s2_val1 = XMLUtil.doReplace(s2, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s3_val1 = XMLUtil.doReplace(s3, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s4_val1 = XMLUtil.doReplace(s4, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s5_val1 = XMLUtil.doReplace(s5, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s6_val1 = XMLUtil.doReplace(s6, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s7_val1 = XMLUtil.doReplace(s7, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      String s8_val1 = XMLUtil.doReplace(s8, "jboss.messaging.datachanneludpaddress", "228.6.6.6");
      
      log.info("s1_val1: " + s1_val1);
      log.info("s2_val1: " + s2_val1);
      log.info("s3_val1: " + s3_val1);
      log.info("s4_val1: " + s4_val1);
      log.info("s5_val1: " + s5_val1);
      log.info("s6_val1: " + s6_val1);
      log.info("s7_val1: " + s7_val1);
      log.info("s8_val1: " + s8_val1);

      assertEquals("ff0e::1:2:3", s1_val1);
      assertEquals("ff0e::1:2:3", s2_val1);
      assertEquals("ff0e::1:2:3", s3_val1);
      assertEquals("ff0e::1:2:3", s4_val1);
      assertEquals("ff0e::1:2:3", s5_val1);
      assertEquals("ff0e::1:2:3", s6_val1);
      assertEquals(s7, s7_val1);
      assertEquals(s8, s8_val1);
      
      System.clearProperty("jboss.messaging.datachanneludpaddress");
      if (existingProp != null)
      {
         System.setProperty("jboss.messaging.datachanneludpaddress", existingProp);
      }
      
   }


}
